#!/usr/bin/env python3

import argparse
import string

TEMPLATE = r"""
\begin{pattern}[S${GLOBALREPR}]{${GLOBALREPR}}{${GLOBALREPR}}
  \poi{sequence}{${SEQUENCE}}
  \poi{global}{${GLOBALREPR}}
  \poi{local}{${LOCALREPR}}
  \poi{start}{\clubsfortwo{A, ${ARIGHT}, ${ALEFT}, r}{B, ${BRIGHT}, ${BLEFT}, r, 12}}
  \poi{throws}{$$A$$ ${ATHROWS}; $$B$$ ${BTHROWS}}
  \poiinterface
  \switchsplit
  \causaldia{
    ${ACAUSAL} |
    .5/${BCAUSAL}}
\end{pattern}
"""


def optimal_rotation(siteswap):
    best_val, best_score = None, 0
    for i in range(len(siteswap)):
        current_val = siteswap[i:] + siteswap[:i]
        current_score = sum(n / (i+1) for i, n in enumerate(current_val))
        if current_val[0] % 2:
            current_score *= 2
        if current_score > best_score:
            best_val = current_val
            best_score = current_score
    return best_val


BASE36_CHARS = {**{i: str(i) for i in range(10)},
                **{(i+10): c for i, c in enumerate(string.ascii_lowercase)}}


def number2throw(num):
    if num == 4:
        return 'h'
    elif num == 2:
        return 'z'
    elif num % 2 == 0:
        count = round((num - 4) / 2)
        if count == 1:
            return 's'
        else:
            return f's$^{count}$'
    else:
        count = round((num - 5) / 2)
        if count == 1:
            return 'p'
        else:
            return f'p$^{count}$'


def number2causal(num):
    if num == 4:
        return 'H'
    elif num == 2:
        return 'Z'
    elif num % 2 == 0:
        count = round((num - 4) / 2)
        if count == 1:
            return 'S'
        else:
            return f'S{count}'
    else:
        count = round((num - 5) / 2)
        if count == 1:
            return 'P'
        else:
            return f'P{count}'


def compute_start_clubs(numeric):
    instructions = numeric * 20
    start_clubs = [True] * len(instructions)
    for idx, instruction in enumerate(instructions):
        effective = idx + instruction - 4
        if 0 <= effective < len(start_clubs):
            start_clubs[effective] = False
    return [sum(start_clubs[hand:-10:4]) + 1 for hand in range(4)]


THROW_NAMES = {
    5: "zaps",
    7: "singles",
    9: "doubles",
    11: "triples",
    13: "quads",
}


def execute(args):
    siteswap = args.siteswap
    numeric = [int(n, 36) for n in siteswap]
    if not args.keep:
        numeric = optimal_rotation(numeric)
    globalrepr = ''.join(map(BASE36_CHARS.get, numeric))
    if len(siteswap) % 2 == 1:
        local_numeric = [numeric[(2*i) % len(numeric)]
                         for i in range(len(numeric))]
        sequence = ' '.join(map(number2throw, local_numeric))
        tmp = list(map(BASE36_CHARS.get, local_numeric))
        tmp.insert((len(tmp) // 2) + 1, '_{B}')
        tmp.insert(0, '{}_{A}')
        localrepr = '$' + ''.join(tmp) + '$'
    else:
        local_numeric_a = [numeric[2*i] for i in range(len(numeric)//2)]
        local_numeric_b = [numeric[2*i+1] for i in range(len(numeric)//2)]
        sequence_a = ' '.join(map(number2throw, local_numeric_a))
        sequence_b = ' '.join(map(number2throw, local_numeric_b))
        sequence = f'$A$: {sequence_a} / $B$: {sequence_b}'
        localrepr_a = "".join(list(map(BASE36_CHARS.get, local_numeric_a)))
        localrepr_b = "".join(list(map(BASE36_CHARS.get, local_numeric_b)))
        localrepr = f'$A$: {localrepr_a} / $B$: {localrepr_b}'
    aright, bright, aleft, bleft = compute_start_clubs(numeric)
    if len(siteswap) % 2 == 1:
        total_casual = list(map(number2causal, 4*numeric))
    elif len(siteswap) % 4 == 2:
        total_casual = list(map(number2causal, 2*numeric))
    else:
        total_casual = list(map(number2causal, numeric))
    if len(siteswap) % 2 == 1:
        total_casual[2*len(numeric)] = ";" + total_casual[2*len(numeric)]
    elif len(siteswap) % 4 == 2:
        total_casual[len(numeric)] = ";" + total_casual[len(numeric)]
    acausal = ' '.join(total_casual[::2]
                       + ['!' + number2causal(numeric[0]), 'N', 'N', 'N'])
    bcausal = ' '.join(total_casual[1::2] + ['N', 'N', 'N'])
    throw_directions = []
    for parity in range(2):
        tmp = []
        for num, throw in THROW_NAMES.items():
            if num in numeric:
                if ((num - 1) // 2 + parity) % 2 == 0:
                    tmp.append(f"crossing {throw}")
                else:
                    tmp.append(f"straight {throw}")
        throw_directions.append(', '.join(tmp))
    athrows, bthrows = throw_directions
    ret = string.Template(TEMPLATE).safe_substitute(
        GLOBALREPR=globalrepr, SEQUENCE=sequence, LOCALREPR=localrepr,
        ARIGHT=aright, ALEFT=aleft, BRIGHT=bright, BLEFT=bleft,
        ATHROWS=athrows, BTHROWS=bthrows, ACAUSAL=acausal, BCAUSAL=bcausal)
    print(ret)
    return ret


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=(
            "Print a LaTeX template from a siteswap."
            " This removes most of the menial tasks of inserting a"
            " siteswap into the anthology."))
    parser.add_argument('siteswap', help="siteswap to convert")
    parser.add_argument('-k', '--keep', action="store_true",
                        help="keep the original rotation of the siteswap")
    args = parser.parse_args()
    execute(args)
